precision highp float;
uniform mat3 uCMSmatrix;
uniform int hdrType;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

vec3 eotf_pq(vec3 color) {
    color = max(color, 0.0);
    color = pow(color, vec3(1.0/78.843750));
    color = max(color - vec3(0.835938), 0.0) / (vec3(18.851562) - vec3(18.687500) * color);
    color = pow(color, vec3(1.0/0.159302));
    return color;
}

vec3 eotf_hlg(vec3 color) {
    float HLG_A = 0.17883277;
    float HLG_B = 0.28466892;
    float HLG_C = 0.55991073;
    vec3 mask = step(vec3(0.5), color);
    color = mask * exp((color - vec3(HLG_C)) * vec3(1.0/HLG_A) + HLG_B) + (1.0 - mask) * vec3(4.0) * color * color;
    color *= vec3(1.0/12.0);
    color = clamp(color, 0.0, 1.0);
    float lum = 0.2627 * color.r + 0.677998 * color.g + 0.059302 * color.b;
    color = vec3(pow(lum, 0.2)) * color;
    return color;
}

float _hable_3_0(float x) {
    return ((x * (0.150000*x + 0.050000)+0.004000)/(x * (0.150000*x + 0.500000) + 0.060000)) - 0.066667;
}

vec3 toneMap(vec3 color, float peakLum) {
    color = color * vec3(peakLum);
    float sig = max(max(color.r, color.g), color.b);
    float sig_peak = peakLum;
    float luma = 0.2126368 * color.r + 0.715183 * color.g + 0.07218021 * color.b;
    float coeff = max(sig - 0.18, 1e-6) / max(sig, 1e-6);
    coeff = pow(coeff, 20.0);
    sig = mix(sig, luma, coeff);
    float sig_orig = max(sig, 1e-6);
    sig = _hable_3_0(sig) / _hable_3_0(sig_peak);
    sig = min(sig, 1.0);
    color *= sig / sig_orig;
    return color;
}

vec4 colorMapping(vec4 color) {
    float peakLum = 0.0;
    //if (hdrType == 1) {
    //    color.rgb = eotf_pq(color.rgb);
    //    peakLum = 110.0;
    //} else {
    //    color.rgb = eotf_hlg(color.rgb);
    //    peakLum = 12.0;
    //}

    color.rgb = eotf_hlg(color.rgb);
    peakLum = 60.0;

    color.rgb = uCMSmatrix * color.rgb;
    color.rgb = toneMap(color.rgb, peakLum);
    color.rgb = pow(color.rgb, vec3(1.0/1.1));
    color.rgb = clamp(color.rgb, 0.0, 1.0);

    return color;
}

void main(void) {
    vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);
    gl_FragColor = colorMapping(textureColor);
}