#extension GL_OES_EGL_image_external : require

precision mediump float;
varying vec2 vTextureCoord;
uniform samplerExternalOES sTexture;
uniform int isHDR;
uniform mat3 uCMSmatrix;
uniform int hdrType;

const float A3 = 0.15;
const float B3 = 0.50;
const float C3 = 0.10;
const float D3 = 0.20;
const float E3 = 0.02;
const float F3 = 0.30;
uniform float uOpacity;

vec3 hableToneMapping(vec3 color) {
    return (color * (color * A3 + B3 * C3) + D3 * E3) / (color * (color * A3 + B3) + D3 * F3) - E3 / F3;
}

float hableF(float inVal) {
    return (inVal * (inVal * A3 + B3 * C3) + D3 * E3) / (inVal * (inVal * A3 + B3) + D3 * F3) - E3 / F3;
}

float ARIB_B67_A = 0.17883277;
float ARIB_B67_B = 0.28466892;
float ARIB_B67_C = 0.55991073;
float arib_b67_inverse_oetf(float x)
{
    x = max(x, 0.0);
    if (x <= (1.0/2.0))
        x = (x * x) * (1.0 / 3.0);
    else
        x = (exp((x - ARIB_B67_C) / ARIB_B67_A) + ARIB_B67_B) / 12.0;
    return x;
}

float arib_b67_ootf(float x)
{
    return x < 0.0 ? x : pow(x, 1.2);
}

float arib_b67_eotf(float x)
{
    return arib_b67_ootf(arib_b67_inverse_oetf(x));
}

float arib_b67_oetf(float x)
{
    x = max(x, 0.0);
    if (x <= (1.0 / 12.0))
        x = sqrt(3.0 * x);
    else
        x = ARIB_B67_A * log(12.0 * x - ARIB_B67_B) + ARIB_B67_C;
    return x;
}

float Lb = 0.1;
float Lw = 302.0;
float sys_gamma = 1.001;
float bfiler(float x) {
    float b = sqrt(3.0 * pow((Lb/Lw), (1.0/sys_gamma)) );
    return max(0.0, ((1.0-b)*x + b) );
}


float ST2084_OOTF_SCALE = 59.49080238715383;

float ST2084_M1 = 0.1593017578125;
float ST2084_M2 = 78.84375;
float ST2084_C1 = 0.8359375;
float ST2084_C2 = 18.8515625;
float ST2084_C3 = 18.6875;


float st_2084_eotf(float x)
{
    if (x > 0.0) {
        float FLT_MIN = 1.17549435082228750797*pow(10.0, -38.0);
        float xpow = pow(x, float(1.0 / ST2084_M2));
        float num = max(xpow - ST2084_C1, 0.0);
        float den = max(ST2084_C2 - ST2084_C3 * xpow, FLT_MIN);
        x = pow(num/den, 1.0 / ST2084_M1);
    } else {
        x = 0.0;
    }

    return x;
}

float hable_f(float value) {
    float a = 0.15;
    float b = 0.50;
    float c = 0.10;
    float d = 0.20;
    float e = 0.02;
    float f = 0.30;
    return (value * (value * a + b * c) + d * e) / (value * (value * a + b) + d * f) - e / f;
}


float REC709_ALPHA = 1.09929682680944;
float REC709_BETA = 0.018053968510807;
float rec_709_oetf(float x)
{
    x = max(x, 0.0);
    if (x < REC709_BETA )
        x = x * 4.5;
    else
        x = REC709_ALPHA * pow(x, 0.45) - (REC709_ALPHA - 1.0);
    return x;
}

float _hable_3_0(float x) {
    return ((x * (0.150000*x + 0.050000) + 0.004000) / (x * (0.150000*x + 0.500000) + 0.060000)) - 0.066667;
}

vec4 _main_4_6(vec4 color) {
     // pl_shader_color_map
     vec3 _src_luma_0_0 = vec3(0.262698,0.678009,0.059293);
     vec3 _dst_luma_1_0 = vec3(0.212637,0.715183,0.072180);
     mat3 _cms_matrix_2_0 = mat3(1.660497,-0.124547,-0.018154,-0.587657,1.132895,-0.100597,-0.072840,-0.008348,1.118751);
     // pl_shader_linearize
     color.rgb = max(color.rgb, 0.0);
     color.rgb = mix(vec3(4.0) * color.rgb * color.rgb,
     exp((color.rgb - vec3(0.559911)) * vec3(1.0/0.178833))
     + vec3(0.284669),
     vec3(lessThan(vec3(0.5), color.rgb)));
     // pl_shader_ootf
     color.rgb = max(color.rgb, 0.0);
     color.rgb *= vec3(0.506970 * pow(dot(_src_luma_0_0, color.rgb), 0.2));
     color.rgb = _cms_matrix_2_0 * color.rgb;
     // pl_shader_tone_map
     float sig = max(max(color.r, color.g), color.b);
     float sig_peak = 12.000000;
     float sig_avg = 0.250000;
     float luma = dot(_dst_luma_1_0, color.rgb);
     float coeff = max(sig - 0.18, 1e-6) / max(sig, 1e-6);
     coeff = pow(coeff, 20.000000);
     color.rgb = mix(color.rgb, vec3(luma), coeff);
     sig = mix(sig, luma, coeff);
     float sig_orig = sig;
     float slope = min(1.0, 0.250000 / sig_avg);
     sig *= slope;
     sig_peak *= slope;
     sig = _hable_3_0(sig) / _hable_3_0(sig_peak);
     sig = min(sig, 1.0);
     color.rgb *= sig / sig_orig;
     // pl_shader_delinearize
     color.rgb = max(color.rgb, 0.0);
     color.rgb = pow(color.rgb, vec3(1.0/2.2));
     return color;
}

vec4 _main_4_0(vec4 color) {
    // pl_shader_color_map
    vec3 _dst_luma_1_0 = vec3(0.212637, 0.715183, 0.072180);
    mat3 _cms_matrix_2_0 = mat3(1.660497, -0.124547, -0.018154,
     -0.587657, 1.132895, -0.100597
    , -0.072840, -0.008348, 1.118751);

    // pl_shader_linearize
    //color.rgb = max(color.rgb, 0.0);
    //color.rgb = pow(color.rgb, vec3(1.0 / 78.843750));
    //color.rgb = max(color.rgb - vec3(0.835938), 0.0) / (vec3(18.851562) - vec3(18.687500) * color.rgb);
    //color.rgb = pow(color.rgb, vec3(1.0 / 0.159302));
    //color.rgb *= vec3(10.000000);
    //color.rgb = _cms_matrix_2_0 * color.rgb;

     color.rgb = max(color.rgb, 0.0);
     color.rgb = mix(vec3(4.0) * color.rgb * color.rgb,
         exp((color.rgb - vec3(0.559911)) * vec3(1.0/0.178833)) + vec3(0.284669),
         vec3(lessThan(vec3(0.5), color.rgb)));


    // pl_shader_tone_map
    float sig = max(max(color.r, color.g), color.b);
    float sig_peak = 100.000000;
    float sig_avg = 0.250000;
    float luma = dot(_dst_luma_1_0, color.rgb);
    float coeff = max(sig - 0.18, 1e-6) / max(sig, 1e-6);
    coeff = pow(coeff, 20.000000);
    color.rgb = mix(color.rgb, vec3(luma), coeff);
    sig = mix(sig, luma, coeff);
    float sig_orig = sig;
    float slope = min(1.0, 0.250000 / sig_avg);
    sig *= slope;
    sig_peak *= slope;
    sig = _hable_3_0(sig) / _hable_3_0(sig_peak);
    sig = min(sig, 1.0);
    color.rgb *= sig / sig_orig;

    // pl_shader_delinearize
    color.rgb = max(color.rgb, 0.0);
    color.rgb = pow(color.rgb, vec3(1.0 / 2.2));
    return color;
}

vec3 eotf_pq(vec3 color) {
    color = max(color, 0.0);
    color = pow(color, vec3(1.0/78.843750));
    color = max(color - vec3(0.835938), 0.0) / (vec3(18.851562) - vec3(18.687500) * color);
    color = pow(color, vec3(1.0/0.159302));
    return color;
}

vec3 ootfForward(vec3 sColor) {
    vec3 color1 = sColor * 12.92;
    vec3 color2 = 1.055 * pow(sColor, vec3(0.4166666666)) - 0.055;
    return mix(color1, color2, step(0.0031308, sColor));
}

vec3 ootfInverse(vec3 sColor) {
    vec3 color1 = sColor / 12.92;
    vec3 color2 = pow((sColor + 0.055) / 1.055, vec3(2.4));
    return mix(color1, color2, step(0.04045, sColor));
}

vec3 eotf_hlg(vec3 color) {
    float HLG_A = 0.17883277;
    float HLG_B = 0.28466892;
    float HLG_C = 0.55991073;
    vec3 mask = step(vec3(0.5), color);
    color = mask * exp((color - vec3(HLG_C)) * vec3(1.0/HLG_A) + HLG_B) + (1.0 - mask) * vec3(4.0) * color * color;
    color *= vec3(1.0/12.0);
    color = clamp(color, 0.0, 1.0);
    float lum = 0.2627 * color.r + 0.677998 * color.g + 0.059302 * color.b;
    color = vec3(pow(lum, 0.2)) * color;
    return color;
}

vec3 toneMap(vec3 color, float peakLum) {
    color = color * vec3(peakLum);
    float sig = max(max(color.r, color.g), color.b);
    float sig_peak = peakLum;
    float luma = 0.2126368 * color.r + 0.715183 * color.g + 0.07218021 * color.b;
    float coeff = max(sig - 0.18, 1e-6) / max(sig, 1e-6);
    coeff = pow(coeff, 20.0);
    sig = mix(sig, luma, coeff);
    float sig_orig = max(sig, 1e-6);
    sig = _hable_3_0(sig) / _hable_3_0(sig_peak);
    sig = min(sig, 1.0);
    color *= sig / sig_orig;
    return color;
}

vec4 colorMapping(vec4 color) {
    float peakLum = 0.0;
    //if (hdrType == 1) {
    //    color.rgb = eotf_pq(color.rgb);
    //    peakLum = 150.0;
    //} else {
    //    color.rgb = eotf_hlg(color.rgb);
    //    peakLum = 12.0;
    //}
     color.rgb = eotf_hlg(color.rgb);
     peakLum = 20.0;

    color.rgb = uCMSmatrix * color.rgb;
    color.rgb = toneMap(color.rgb, peakLum);
    color.rgb = pow(color.rgb, vec3(1.0/2.2));
    color.rgb = clamp(color.rgb, 0.0, 1.0);
    return color;
}


   precision mediump int; 
         precision mediump float; 
         varying vec2 vTextureCoord; 
         uniform sampler2D uBitmap1;  
         uniform float uOpacity;  
         float _hable_3_0(float x) {  
         return ((x * (0.150000*x + 0.050000)+0.004000)/(x * (0.150000*x + 0.500000) + 0.060000)) - 0.066667; 
         }  
         vec4 _main_4_0(vec4 color) { 
         // pl_shader_color_map 
         vec3 _src_luma_0_0 = vec3(0.262698,0.678009,0.059293); 
         vec3 _dst_luma_1_0 = vec3(0.212637,0.715183,0.072180); 
         mat3 _cms_matrix_2_0 = mat3(1.660497,-0.124547,-0.018154,-0.587657,1.132895,-0.100597,-0.072840,-0.008348,1.1
         18751); 
         // pl_shader_linearize            
         color.rgb = max(color.rgb, 0.0);  
         color.rgb = mix(vec3(4.0) * color.rgb * color.rgb, 
         exp((color.rgb - vec3(0.559911)) * vec3(1.0/0.178833))  
         + vec3(0.284669),         
         vec3(lessThan(vec3(0.5), color.rgb)));    
         // pl_shader_ootf        
         color.rgb = max(color.rgb, 0.0);  
         color.rgb *= vec3(0.506970 * pow(dot(_src_luma_0_0, color.rgb), 0.2)); 
         color.rgb = _cms_matrix_2_0 * color.rgb; 
         // pl_shader_tone_map 
         float sig = max(max(color.r, color.g), color.b);   
         float sig_peak = 12.000000;                
         float sig_avg = 0.250000;                    
         float luma = dot(_dst_luma_1_0, color.rgb);  
         float coeff = max(sig - 0.18, 1e-6) / max(sig, 1e-6);  
         coeff = pow(coeff, 20.000000);         
         color.rgb = mix(color.rgb, vec3(luma), coeff);  
         sig = mix(sig, luma, coeff);     
         float sig_orig = sig; 
         float slope = min(1.0, 0.250000 / sig_avg);  
         sig *= slope;               
         sig_peak *= slope;            
         sig = _hable_3_0(sig) / _hable_3_0(sig_peak); 
         sig = min(sig, 1.0);         
         color.rgb *= sig / sig_orig;  
         // pl_shader_delinearize          
         color.rgb = max(color.rgb, 0.0);  
         color.rgb = pow(color.rgb, vec3(1.0/2.2)); 
         return color; 
         } 
         void main()  
         {  
         vec4 Coefficients[4]; 
         vec4 sColor = texture2D(uBitmap1, vTextureCoord);  
         
         Coefficients[0]=vec4(1.164384,1.164384,1.164384,1.164384); 
         Coefficients[1]=vec4(0.000000,-0.187326,2.141772,0.000000); 
         Coefficients[2]=vec4(1.678674,-0.650424,0.000000,1.678674); 
         Coefficients[3]=vec4(-0.915688,0.347459,-1.148145,-0.915688); 
          
         sColor = vec4(sColor.rgb, 1.0);  
         vec4 color0 = vec4(sColor.r, sColor.r,sColor.r,1.0); 
         vec4 color1 = vec4(sColor.g, sColor.g,sColor.g,1.0); 
         vec4 color2 = vec4(sColor.b, sColor.b,sColor.b,1.0); 
         vec4 result = (color0 * Coefficients[0]) + Coefficients[3]; 
          result = (color1 * Coefficients[1]) + result; 
         result = (color2 * Coefficients[2]) + result; 
          result = _main_4_0(result); 
         gl_FragColor = result; 
         } )
         
void main ()
{
    vec4 textureColor = texture2D(sTexture, vTextureCoord);
    vec4 color = textureColor;

    color = _main_4_0(color);
    //color.rgb = ootfInverse(color.rgb);
    if (isHDR == 1) {
        //color = colorMapping(color);
    }

    gl_FragColor = color;
}