#extension GL_OES_EGL_image_external : require

precision mediump float;
varying vec2 vTextureCoord;
uniform samplerExternalOES sTexture;
uniform int isHDR;
uniform mat3 uCMSmatrix;

float A3 = 0.15;
float B3 = 0.50;
float C3 = 0.10;
float D3 = 0.20;
float G3 = 0.02;
float F3 = 0.30;
vec3 hableToneMapping(vec3 color) {
    return (color * (color * A3 + B3 * C3) + D3 * G3) / (color * (color * A3 + B3) + D3 * F3) - G3 / F3;
}

float hableF(float inVal) {
    return (inVal * (inVal * A3 + B3 * C3) + D3 * G3) / (inVal * (inVal * A3 + B3) + D3 * F3) - G3 / F3;
}

float ARIB_B67_A = 0.17883277;
float ARIB_B67_B = 0.28466892;
float ARIB_B67_C = 0.55991073;
float arib_b67_inverse_oetf(float x)
{
    x = max(x, 0.0);
    if (x <= (1.0/2.0))
        x = (x * x) * (1.0 / 3.0);
    else
        x = (exp((x - ARIB_B67_C) / ARIB_B67_A) + ARIB_B67_B) / 12.0;
    return x;
}

float arib_b67_ootf(float x)
{
    return x < 0.0 ? x : pow(x, 1.2);
}

float arib_b67_eotf(float x)
{
    return arib_b67_ootf(arib_b67_inverse_oetf(x));
}

float arib_b67_oetf(float x)
{
    x = max(x, 0.0);
    if (x <= (1.0 / 12.0))
        x = sqrt(3.0 * x);
    else
        x = ARIB_B67_A * log(12.0 * x - ARIB_B67_B) + ARIB_B67_C;
    return x;
}

float Lb = 0.1;
float Lw = 302.0;
float sys_gamma = 1.001;
float bfiler(float x) {
    float b = sqrt(3.0 * pow((Lb/Lw), (1.0/sys_gamma)) );
    return max(0.0, ((1.0-b)*x + b) );
}

float ST2084_OOTF_SCALE = 59.49080238715383;
float ST2084_M1 = 0.1593017578125;
float ST2084_M2 = 78.84375;
float ST2084_C1 = 0.8359375;
float ST2084_C2 = 18.8515625;
float ST2084_C3 = 18.6875;

float A = 0.15;
float B = 0.50;
float C = 0.10;
float D = 0.20;
float E = 0.02;
float F = 0.30;
float W = 11.2;

vec3 Uncharted2Tonemap(vec3 x)
{
    return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F;
}

float st_2084_eotf(float x)
{
    if (x > 0.0) {
        float FLT_MIN = 1.17549435082228750797*pow(10.0, -38.0);
        float xpow = pow(x, float(1.0 / ST2084_M2));
        float num = max(xpow - ST2084_C1, 0.0);
        float den = max(ST2084_C2 - ST2084_C3 * xpow, 0.0);
        x = pow(num/den, 1.0 / ST2084_M1);
    } else {
        x = 0.0;
    }

    return x;
}

float REC709_ALPHA = 1.09929682680944;
float REC709_BETA = 0.018053968510807;
float rec_709_oetf(float x)
{
    x = max(x, 0.0);
    if (x < REC709_BETA )
        x = x * 4.5;
    else
        x = REC709_ALPHA * pow(x, 0.45) - (REC709_ALPHA - 1.0);
    return x;
}

float rec_1886_eotf(float x)
{
    return x < 0.0 ? 0.0 : pow(x, 2.4);
}

float ootf_st2084(float x)
{
    return rec_1886_eotf(rec_709_oetf(x * ST2084_OOTF_SCALE)) / 100.0;
}

float st_2084_inverse_eotf(float x)
{
    // Filter negative values to avoid NAN, and also special-case 0 so that (f(g(0)) == 0).
    if (x > 0.0) {
        float xpow = pow(x, ST2084_M1);

        float num = (ST2084_C1 - 1.0) + (ST2084_C2 - ST2084_C3) * xpow;
        float den = 1.0 + ST2084_C3 * xpow;
        x = pow(1.0 + num / den, ST2084_M2);
    } else {
        x = 0.0;
    }

    return x;
}

float st_2084_oetf(float x)
{
    return st_2084_inverse_eotf(ootf_st2084(x));
}

float hable_f(float value) {
    float a = 0.15;
    float b = 0.50;
    float c = 0.10;
    float d = 0.20;
    float e = 0.02;
    float f = 0.30;
    return (value * (value * a + b * c) + d * e) / (value * (value * a + b) + d * f) - e / f;
}

#define FFMAX(a,b) ((a) > (b) ? (a) : (b))

float gamma = 0.8;
vec3 toLinear(vec3 colour) { return pow(colour, vec3(gamma)); }

vec3 toHDR(in vec3 colour, float range) {
    return toLinear(colour) * range;
}

vec3 tone_map(vec3 color, float exp)
{
    return 1.0 - exp2(-color * exp);
}

vec3 YuvConvertRGB_BT2020(vec3 yuv, int normalize) {
    vec3 rgb;
    int y = int(yuv.x);
    int u = int(yuv.y);
    int v = int(yuv.z);
    // [64, 960]
    float r = float(y - 64) * 1.164384                             - float(v - 512) * -1.67867;
    float g = float(y - 64) * 1.164384 - float(u - 512) * 0.187326 - float(v - 512) * 0.65042;
    float b = float(y - 64) * 1.164384 - float(u - 512) * -2.14177;
    rgb.r = r;
    rgb.g = g;
    rgb.b = b;
    if (normalize == 1) {
        rgb.r = r / 1024.0;
        rgb.g = g / 1024.0;
        rgb.b = b / 1024.0;
    }

    return rgb;
}

void main ()
{
    vec4 textureColor = texture2D(sTexture, vTextureCoord);
    vec4 color = textureColor;
    vec3 rgb10bit = color.rgb;
    vec3 fragColor = color.rgb;

    //color = _main_4_6(color);
    //color.rgb = ootfInverse(color.rgb);
    if (isHDR == 1) {
        float ST2084_PEAK_LUMINANCE = 10000.0;
        float peak_luminance = 100.0;
        // 参考zscale源代码参数ST2084_PEAK_LUMINANCE固定1w，peak_luminance视频元数据-标峰亮度值，静态元数据的全视频用一个peak，动态元数据的每一帧一个peak
        // 参考zscale源代码to_linear的GammaOperationC，to_linear_scale赋值是postscale，后处理缩放。
        float to_linear_scale = ST2084_PEAK_LUMINANCE / peak_luminance;
        fragColor = to_linear_scale * vec3(st_2084_eotf(color.r), st_2084_eotf(color.g), st_2084_eotf(color.b));

        // 第二步、HDR线性 ToneMapping映射转成 SDR线性
        // 参考ffmpeg的tonemap函数，hable算法。
        highp float sig;
        highp float sig_orig;
        sig = FFMAX(FFMAX(FFMAX(fragColor.r, fragColor.g), fragColor.b), 1e-6);
        sig_orig = sig;
        float peak = 540.0 / 100.0; // 手机设备的最大亮度值MaxCLL / REFERENCE_WHITE(固定100);
        sig = hable_f(sig) / hable_f(peak);
        fragColor.r = fragColor.r * (sig / sig_orig);
        fragColor.g = fragColor.g * (sig / sig_orig);
        fragColor.b = fragColor.b * (sig / sig_orig);

        // 第三步、逆线性光信号，变回电
        // 参考zscale源代码，rec_709的to_gamma，没有prescale和postscale.
        fragColor = vec3(rec_709_oetf(fragColor.r), rec_709_oetf(fragColor.g), rec_709_oetf(fragColor.b));
    }

    gl_FragColor.rgb = fragColor;
    gl_FragColor.a = color.a;
}